#! /bin/sh
# RWiki and depending libraries installer
# copyright (c) 2000 ZnZ <zn@mbf.nifty.com>

WORK=/tmp
WGET='wget -N'
PREFIX=${HOME}
LIB=${PREFIX}/lib/ruby
BIN=${PREFIX}/bin
RWIKIPARENTDIR=${PREFIX}
CGIDIR=${PREFIX}/public_html
CGINAME=rw.cgi

set -e

PATH=${BIN}:$PATH
RUBYLIB_ORG=$RUBYLIB
RUBYLIB=${LIB}:$RUBYLIB
export RUBYLIB

if [ -z "$EDITOR" ]; then
  EDITOR=vi
fi

if ruby -e 'exit RUBY_VERSION < "1.6" ? 0 : 1'; then
  echo You have to install ruby/1.6 series.
  exit 0
fi

for DIR in ${WORK}/rwiki-installer $LIB $BIN $RWIKIDIR $CGIDIR; do
  if [ ! -d $DIR ]; then
    mkdir -p $DIR || exit 1
  fi
done

# enter working directory
WORK=${WORK}/rwiki-installer
cd $WORK || exit 2


# define functions begin

raaget () {
  NAME=$1
  URL=$2

  if [ ! -f raa-${NAME}.html ]; then
    $WGET -O raa-${NAME}.html $URL || exit 3
  fi
  URL=`grep tar.gz raa-${NAME}.html|sed 's/<[^>][^>]*>//g'`
  FILENAME=`basename ${URL}`
  if [ ! -f $FILENAME ]; then
    $WGET $URL || exit 4
  fi

  zcat $FILENAME | tar -xf -
}

changedir () {
  for DIR in ${1}*; do
    if [ -d $DIR ]; then
      cd $DIR || exit 5
      return
    fi
  done
}

cpfile () {
  changedir $1
  cp -fp $2 $LIB || exit 7
  cd ..
}

rsetup () {
  changedir $1
  ruby setup.rb config --bin-dir=$BIN --rb-dir=$LIB --so-dir=$LIB || exit 8
  ruby setup.rb setup   || exit 9
  ruby setup.rb install || exit 10
  ruby setup.rb clean   || exit 11
  rm -rf config.save ext/cparse/Makefile ext/strscanso/Makefile
  cd ..
}

libcheck () {
  ruby -e "begin;require '$1';rescue LoadError;exit 1;end"
}

# define functions end


# library install begin

if which racc>/dev/null 2>&1; then
  echo racc already installed.
else
  echo start racc installation.
  raaget racc 'http://www.ruby-lang.org/en/raa-list.rhtml?name=Racc'
  rsetup racc
  echo finish racc installation.
fi

if libcheck 'strscan'; then
  echo strscan already installed.
else
  echo start strscan installation.
  raaget strscan 'http://www.ruby-lang.org/en/raa-list.rhtml?name=strscan'
  rsetup strscan
  echo finish strscan installation.
fi

if libcheck 'optparse'; then
  echo OptionParser already installed.
else
  echo start OptionParser installation.
  raaget optparse 'http://www.ruby-lang.org/en/raa-list.rhtml?name=OptionParser'
  cpfile optparse optparse.rb
  echo finish OptionParser installation.
fi

if libcheck 'rd/rdfmt'; then
  echo RDtool already installed.
else
  echo start RDtool installation.
  raaget rdtool 'http://www.ruby-lang.org/en/raa-list.rhtml?name=RDtool'

  changedir rdtool
  ruby rdtoolconf.rb || exit 12
  cp Makefile Makefile.orig || exit 13
  awk "/^BIN_DIR/{print \"BIN_DIR = $BIN\";next} /^SITE_RUBY/{print \"SITE_RUBY = $LIB\";next} /^RD_DIR/{print \"RD_DIR = ${LIB}/rd\";next} {print}" Makefile.orig > Makefile || exit 14
  rm Makefile.orig || exit 15
  # cp missing/rd/* rd
  make install install-rmi2html || exit 16
  make clean || exit 17
  cd ..
  echo finish RDtool installation.
fi

# library install end


# RWiki install begin

echo start RWiki installation.
cd $RWIKIPARENTDIR || exit 18
raaget rwiki 'http://www.ruby-lang.org/en/raa-list.rhtml?name=RWiki'
changedir rwiki
RWIKI_DIR=`pwd`
echo -n Configure rw-config.rb. [Press Enter]
cp rw-config.rb rw-config.rb.orig
awk "/^  MAILTO/{sub(/'[^']+'/,\"'mailto:`whoami`@`hostname`'\")}/^  ADDRESS/{sub(/'[^']+'/,\"'`whoami`'\")}/^  CGI_NAME/{sub(/'[^']+'/,\"'$CGINAME'\")}{print}" rw-config.rb.orig > rw-config.rb
read
$EDITOR rw-config.rb || exit 19

# set DRB_URI
eval `grep druby rw-config.rb|sed 's/ //g;s|druby://:|druby://localhost:|'`

# install rw-cgi.rb to ${CGIDIR}/${CGINAME}
awk "NR==1{print \"#!`which ruby`\";next} /^.RWIKI_DIR/{sub(/File[^)]+\\)/,\"'$RWIKI_DIR'\")} /^.RWIKI_URI/{sub(/'[^']+'/,\"'$DRB_URI'\")} {print}" rw-cgi.rb > ${CGIDIR}/$CGINAME
chmod +x ${CGIDIR}/$CGINAME || exit 20
echo -n Configure ${CGINAME}. Comment 'SETUP' must be configured. [Press Enter]
read
$EDITOR ${CGIDIR}/${CGINAME} || exit 21
echo finish RWiki installation.

# RWiki install end


# check original $RUBYLIB
RUBYLIB=$RUBYLIB_ORG
if ruby -e 'exit($:.include?(ARGV[0])||$:.include?(File.expand_path(ARGV[0]))?1:0)' $LIB
then
  echo Exec either of below to set the search path.
  if [ -z "$RUBYLIB" ]; then
    echo setenv RUBYLIB $LIB
    echo export RUBYLIB="$LIB"
  else
    echo setenv RUBYLIB ${LIB}:\$RUBYLIB
    echo export RUBYLIB=${LIB}:\$RUBYLIB
  fi
fi

echo Execute rwiki server.
echo \$ cd $RWIKI_DIR\; ruby -Ke -I./lib rwiki.rb
echo Open $CGINAME with your web browser.
echo Exec below for remove work files.
echo \$ rm -rf ${WORK}/rwiki-installer
echo \$ rm ${RWIKIPARENTDIR}/raa-rwiki.html ${RWIKIPARENTDIR}/rwiki-*.tar.gz
